package cz.muni.fi.pv168.graves;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.sql.DataSource;

/**
 * This is example of an JDBC implementation of some methods.
 *
 *
 * To create a table for graves:
 *
 * CREATE TABLE GRAVE
 *   (ID INTEGER NOT NULL PRIMARY KEY GENERATED ALWAYS AS IDENTITY,
 *   GRAVECOLUMN INTEGER,
 *   GRAVEROW INTEGER,
 *   CAPACITY INTEGER,
 *   NOTE VARCHAR(50)
 * );
 *
 * JDBC (derby.jar, derbyclient.jar, derbynet.jar) Driver
 * must be available on the classpath.
 * Can be downloaded from:
 * http://db.apache.org/derby/releases/release-10.7.1.1.cgi
 * To add: Libraries ->right click ->Add JAR/Folder ->select these 3 jars
 *
 * Also commons-dbcp.jar and commons-pool.jar must be added.
 * http://commons.apache.org/dbcp/download_dbcp.cgi
 * http://commons.apache.org/pool/download_pool.cgi
 *
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @author Petr Adamek (xadamek2@fi.muni.cz)
 */
public class CemeteryManagerImpl implements CemeteryManager {

    private static final Logger logger = Logger.getLogger(
            CemeteryManagerImpl.class.getName());
    private DataSource ds;

    void setDataSource(DataSource ds) {
        this.ds = ds;
    }

    public List<Grave> getAllGraves() {
        Connection conn = null;
        try {
            conn = ds.getConnection();
            PreparedStatement st = conn.prepareStatement("SELECT * from grave");
            ResultSet rs = st.executeQuery();
            List<Grave> found = new ArrayList<Grave>();
            while (rs.next()) {
                found.add(rowToGrave(rs));
            }
            return found;
        } catch (SQLException ex) {
            logger.log(Level.SEVERE, "Error when getting all graves from DB", ex);
            throw new RuntimeException("Error when getting all graves from DB", ex);
        } finally {
            DBUtils.closeQuietly(conn);
        }
    }

    public Grave getGraveById(Long id) {
        if (id == null) {
            throw new IllegalArgumentException("id");
        }
        Connection conn = null;
        try {
            conn = ds.getConnection();
            PreparedStatement st = conn.prepareStatement(
                    "SELECT id, graverow,gravecolumn,capacity,note FROM GRAVE WHERE ID = ?");
            st.setLong(1, id);

            ResultSet rs = st.executeQuery();

            if (rs.next()) {
                Grave result = rowToGrave(rs);
                assert !rs.next();
                return result;
            } else {
                return null;
            }

        } catch (SQLException ex) {
            logger.log(Level.SEVERE, "Error when getting grave from DB", ex);
            throw new RuntimeException("Error when getting grave from DB", ex);
        } finally {
            DBUtils.closeQuietly(conn);
        }
    }

    public void addGrave(Grave grave) {
        if (grave == null) {
            throw new IllegalArgumentException("grave");
        }
        if (grave.getId() != null) {
            throw new IllegalArgumentException("grave id is already set");
        }
        if (grave.getCapacity() <= 0) {
            throw new IllegalArgumentException("grave capacity is negative");
        }

        Connection conn = null;
        try {
            conn = ds.getConnection();
            PreparedStatement st = conn.prepareStatement(
                    "INSERT INTO GRAVE (capacity,graverow,gravecolumn,note) VALUES (?,?,?,?)",
                    Statement.RETURN_GENERATED_KEYS);
            st.setInt(1, grave.getCapacity());
            st.setInt(2, grave.getRow());
            st.setInt(3, grave.getColumn());
            st.setString(4, grave.getNote());

            int count = st.executeUpdate();
            assert count == 1;

            Long id = getId(st.getGeneratedKeys());
            grave.setId(id);

        } catch (SQLException ex) {
            logger.log(Level.SEVERE, "Error when inserting grave into DB", ex);
            throw new RuntimeException("Error when inserting into DB", ex);
        } finally {
            DBUtils.closeQuietly(conn);
        }

    }

    public void deleteGrave(Grave grave) {
        if (grave == null) {
            throw new IllegalArgumentException("grave");
        }
        if (grave.getBodies() != null && !grave.getBodies().isEmpty()) {
            throw new IllegalArgumentException("can't delete non-empty grave");
        }
        Connection conn = null;
        try {
            conn = ds.getConnection();
            PreparedStatement st = conn.prepareStatement("DELETE FROM grave WHERE id=?");
            st.setLong(1, grave.getId());
            if (st.executeUpdate() == 0) {
                throw new IllegalArgumentException("grave not found");
            }
        } catch (SQLException ex) {
            logger.log(Level.SEVERE, "Error when deleting grave from DB", ex);
            throw new RuntimeException("Error when deleting grave from DB", ex);

        } finally {
            DBUtils.closeQuietly(conn);
        }


    }

    public void updateGrave(Grave grave) {
        throw new UnsupportedOperationException("method not implemented yet");
    }

    private Grave rowToGrave(ResultSet rs) throws SQLException {
        Grave result = new Grave();
        result.setId(rs.getLong("id"));
        result.setCapacity(rs.getInt("capacity"));
        result.setColumn(rs.getInt("gravecolumn"));
        result.setRow(rs.getInt("graverow"));
        result.setNote(rs.getString("note"));
        return result;
    }

    private static Long getId(ResultSet keys) throws SQLException {
        if (keys.getMetaData().getColumnCount() != 1) {
            throw new IllegalArgumentException("Given ResultSet contains more columns");
        }
        if (keys.next()) {
            Long result = keys.getLong(1);
            if (keys.next()) {
                throw new IllegalArgumentException("Given ResultSet contains more rows");
            }
            return result;
        } else {
            throw new IllegalArgumentException("Given ResultSet contain no rows");
        }
    }
}
